unit GIDPanel;

{ TGray_If_Disabled_Panel
  Version 0.2  1997/Mar/01  (C) 1997 Christoph R. Kirchner

Description:

  TGray_If_Disabled_Panel is a TPanel-decendent that gets shaded if
  Enabled = false.
  It is made for Delphi 2.0 but can be used with Delphi 1.0 also (the sample
  does not work and you have to remove GIDPanel.dcr before compiling the 
  libary).


Properties:

  DisableAll: Boolean
    If true everything on the panel gets shaded,
    else only the background of the panel gets shaded.


Known bugs:

  - After the repaint of components of the shaded panel, the shading is not
    always redrawn. This happens if the component is owned by another
    component (e.g. a panel) in the TGray_If_Disabled_Panel.
}
{ Users of this unit must accept this disclaimer of  warranty:
    "This unit is supplied as is. The author disclaims all warranties, expressed
    or implied, including, without limitation, the warranties of merchantability
    and of fitness for any purpose.
    The author assumes no liability for damages, direct or consequential, which
    may result from the use of this unit."

  This Unit is donated to the public as public domain.

  This Unit can be freely used and distributed in commercial and private
  environments provided this notice is not modified in any way.

  If you do find this Unit handy and you feel guilty for using such a great
  product without paying someone - sorry.

  Please forward any comments or suggestions to Christoph Kirchner at:
  ckirchner@geocities.com
}
{ Maybe you can find an update of this component at:
  http://www.geocities.com/SiliconValley/Heights/7874/index.html
}


interface

uses
  WinTypes, WinProcs, {Windows,} Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type

  TGray_If_Disabled_Panel = class(TPanel)
  private
    FTopPanel: TPanel;
    FDisableAll: Boolean;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property DisableAll: Boolean read FDisableAll write FDisableAll;
  end;


{ Invisible panel: }

  TTransparentPanel = class(TPanel)
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WMEraseBkgnd(var Message: TWmEraseBkgnd);
              message WM_ERASEBKGND;
  end;



procedure Register;


implementation

uses Libconst;

type

  TTopPanel = class(TTransparentPanel)
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;


procedure ShadeRect(DC: HDC; const Rect: TRect);
const
  HatchBits: array[0..7] of Word = ($11, $22, $44, $88, $11, $22, $44, $88);
var
  Bitmap: HBitmap;
  SaveBrush: HBrush;
  SaveTextColor, SaveBkColor: TColorRef;
begin
  Bitmap := CreateBitmap(8, 8, 1, 1, @HatchBits);
  SaveBrush := SelectObject(DC, CreatePatternBrush(Bitmap));
  SaveTextColor := SetTextColor(DC, clWhite);
  SaveBkColor := SetBkColor(DC, clBlack);
  with Rect do PatBlt(DC, Left, Top, Right - Left, Bottom - Top, $00A000C9);
  SetBkColor(DC, SaveBkColor);
  SetTextColor(DC, SaveTextColor);
  DeleteObject(SelectObject(DC, SaveBrush));
  DeleteObject(Bitmap);
end;


constructor TTopPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Parent := TWinControl(AOwner);
  Align := alClient;
  BevelInner := bvNone;
  BevelOuter := bvNone;
  Caption := '';
end;

procedure TTopPanel.Paint;
begin
  ShadeRect(Canvas.Handle, GetClientRect);
end;



procedure TTransparentPanel.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := Params.ExStyle or WS_EX_TRANSPARENT;
end;

procedure TTransparentPanel.WMEraseBkgnd(var Message: TWmEraseBkgnd);
begin  { do not erase background: }
  Message.Result := 1; { we have done anything that was to do }
end;




constructor TGray_If_Disabled_Panel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTopPanel := nil;
  FDisableAll := true;
end;

procedure TGray_If_Disabled_Panel.Paint;
begin
  inherited Paint;
  if not Enabled then
    if (not DisableAll) or (csDesigning in ComponentState) then
      ShadeRect(Canvas.Handle, GetClientRect)
    else
      if Assigned(FTopPanel) then
        FTopPanel.Repaint;
end;

procedure TGray_If_Disabled_Panel.WMCommand(var Message: TWMCommand);
begin
  inherited;
  if (Message.Ctl <> 0) then
    if Assigned(FTopPanel) then
      FTopPanel.Repaint;
end;

procedure TGray_If_Disabled_Panel.CMEnabledChanged(var Message: TMessage);
begin
  if (not DisableAll) or (csDesigning in ComponentState) then
  begin
    Invalidate;
    exit;
  end;
  if Enabled then
  begin
    if Assigned(FTopPanel) then
    begin
      FTopPanel.Parent := nil;
      FTopPanel.Free;
      FTopPanel := nil;
    end;
    Invalidate;
  end
  else
  begin
    if Assigned(FTopPanel) then
      exit;
    FTopPanel := TTopPanel.Create(self);
{    FTopPanel.Enabled := false; }
  end;
end;


procedure Register;
begin
  RegisterComponents(LoadStr(srStandard), [TGray_If_Disabled_Panel]);
{ or at LoadStr(srAdditional) ? }
end;



end.
